/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2019, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Created by: XIAO Tong (xiaotong@mail.neu.edu.cn) 2019-03-27
 */

#include "T2TSearch.h"

using namespace nts;

namespace transformer
{

/* 
search for the most promising states 
>> model - the transformer model
>> input - input of the model
>> padding - padding of the input
>> output - output that represents the sequences as rows
*/
void T2TSearch::Search(T2TModel * model, XTensor * input, XTensor * padding, XTensor * output)
{
    XTensor maskEnc;
    XTensor encoding;

    /* encoder mask */
    model->MakeMTMaskEnc(*input, *padding, maskEnc);
    
    /* make the encoding network */
    encoding = model->MakeEncoder(*input, maskEnc, false);
    encoding.SetName(ENCODING_NAME);

    T2TPredictor predictor;
    T2TStateBundle state1, state2;
    T2TStateBundle * cur = &state1;
    T2TStateBundle * next = &state2;

    /* initialize the predictor */
    predictor.Init(model, &encoding, cur);

    /* generate the sequence from left-to-right */
    for(int i = 0 ; i < maxLength; i++){

        /* read the current state */
        predictor.Read(model, cur);

        /* predict the next state */
        predictor.Predict(next);

        /* pruning */
        Prune(next);

        T2TStateBundle * backup = cur;
        cur = next;
        next = backup;
    }
}

/* 
beam pruning 
>> beam - the beam that keeps a number of states
*/
void T2TSearch::Prune(T2TStateBundle * beam)
{
}

/* 
save the output sequences in a tensor 
>> beam - 
*/
void T2TSearch::DumpOutput(T2TStateBundle * beam, XTensor * output)
{
}

}
